API Reference    {#mathfu_api_reference}
=============

This document describes all classes, functions, macros and files that make
up the [MathFu][] library.  This is intended to be used as a reference for
C++ programmers who are familiar with [geometry][], [vectors][], [matrices][],
[quaternions][] and [linear algebra][].

The API reference is broken into the following sections:
   * [Build Configuration](@ref mathfu_build_config)
      - Describes configuration options for [MathFu][] code generation.
   * [Allocators](@ref mathfu_allocator)
      - Allocators to simplify the process of memory allocation for [MathFu][]
        data structures.
   * [Constants](@ref mathfu_constants)
      - Constants to use in conjunction with [MathFu][] classes.
   * [Vectors](@ref mathfu_vector)
      - Class and functions to manipulate [vectors][].
   * [Matrices](@ref mathfu_matrix)
      - Class and functions to manipulate [matrices][].
   * [Quaternions](@ref mathfu_quaternion)
      - Class and functions to manipulate [quaternions][].
   * [GLSL Mappings](@ref mathfu_glsl)
      - Mappings to GLSL data types and functions.
   * [Utility Functions](@ref mathfu_utilities)
      - Variety of useful functions that can be used with
        [Vector](@ref mathfu::Vector) and [Matrix](@ref mathfu::Matrix)
        classes.
   * [Version Constants](@ref mathfu_version)
      - Constants which identify the library version.

   [MathFu]: @ref mathfu_overview
   [geometry]: http://en.wikipedia.org/wiki/Geometry
   [vectors]: http://en.wikipedia.org/wiki/Euclidean_vector
   [matrices]: http://en.wikipedia.org/wiki/Matrix_(mathematics)
   [quaternions]: http://en.wikipedia.org/wiki/Quaternion
   [linear algebra]: http://en.wikipedia.org/wiki/Linear_algebra
